#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <avr/pgmspace.h>   /*   usbdrv.h */
#include "usbdrv.h"

typedef unsigned char byte;

#define RQ_DATA_READY     0x31
#define RQ_DATA_READ      0x32
#define RQ_END_FLAG_READ  0x33

byte dataBuffer[4];                 //     
byte dataReady, endFlag;            //        
byte w, ww, www;                    //  

//=============================================================================
//  USB
//=============================================================================

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
  usbRequest_t    *rq = (void *)data;
  
  switch(rq->bRequest){
    //---------------------------
    case RQ_DATA_READY:             //    
      dataBuffer[3] = dataReady;    //  -    
      usbMsgPtr = dataBuffer + 3;
      return 1;
    //---------------------------
    case RQ_DATA_READ:              //    
      usbMsgPtr = dataBuffer;
      PORTB &= ~_BV(PB0);           //   (   AVR-USB-MEGA16)
      dataReady = 0;                //     (..   )
      return 3;
    //---------------------------
    case RQ_END_FLAG_READ:          //     
      dataBuffer[3] = endFlag;
      usbMsgPtr = dataBuffer + 3;
//      endFlag = 0;
      return 1;
  }
  return 0;
}

//=============================================================================
//    ,    INT1. 
//      :
// 1.  15    
// 2.    
// 3.         , 
//       INT2
//=============================================================================
ISR(INT1_vect)
{
  dataBuffer[0] = PINA;             //         A
                                    //       B, C.    -   D.
                                    //     AVR-USB-MEGA16 -    A  .
                                    //   :
                                    //  2^0	                    -	PA0
                                    //  2^1	                    -	PA1
                                    //  2^2	                    - PA2
                                    //  2^3	                    -	PA3
                                    //  2^4	                    -	PA4
                                    //  2^5	                    -	PA5
                                    //  2^6	                    -	PA6
                                    //  2^7	                    -	PA7
                                    //  2^8	                    -	PB1
                                    //  2^9	                    -	PB3
                                    //  2^10	                  -	PB4
                                    //  2^11	                  -	PC0
                                    //  2^12	                  -	PC1
                                    //  2^13	                  -	PC6
                                    //  2^14	                  -	PC7
                                    //  	                  -	PD0
                                    //  	          -	PD1
  //        
  ww = PINC; www = ww; ww >>= 1; www <<= 3; w  = ((ww & 0b01100000) | (www & 0b00011000));
  ww = PINB; www = ww; ww >>= 2; www >>= 1; w |= ((ww & 0b00000110) | (www & 0b00000001));
  ww = PIND; www = ww; www <<= 7; dataBuffer[1] = (www & 0b10000000) | w; //  
  dataBuffer[2] = ((ww >> 1) & 0b00000001); //       -  
  
  PORTB |= _BV(PB0);                //   (  )
  dataReady = 0x01;                 //    
  if(endFlag)                       //       -
    endFlag = 0;                    //    
}

//=============================================================================
//         -  .
//       INT2.   
//    
//=============================================================================
ISR(INT2_vect)
{
  endFlag = 1;              //    
}

//=============================================================================
//    USB,     
//   ,      
//   INT1  INT2
//=============================================================================
int main(void)
{
  uchar   i;

  wdt_enable(WDTO_1S);
  /*    watchdog,   .   
      watchdog (on/off, period)   !
  */
  /*  RESET:          (pull-up).
      ,    D+ and D-,  ,      
     .
  */
  usbInit();              // . usbdrv.h  usbdrv.c
  usbDeviceDisconnect();  /* . usbdrv.h -  ,  ,   ! */
  i = 0;
  while(--i) {            /*  USB disconnect   > 250 ms */
    wdt_reset();
    _delay_ms(1);
  }
  usbDeviceConnect();   // . usbdrv.h

  DDRA    = 0x00;       //   A -  
  PORTA   = 0xFF;       //   

  DDRB   |= _BV(PB0);   //  0  B -   
  PORTB  &= ~_BV(PB0);
  DDRB   &= 0xE1;       //  1-4  B -  
  PORTB  |= 0x1E;       //   
  
  DDRC   &= 0x3C;       //  0,1,6,7  C -  
  PORTC  |= 0xC3;       //   
  
  DDRD   &= 0x14;       //  0,1,3,5,6,7  D -  
  PORTD  |= 0xEB;       //   
  
  //     -    : 1->ISC11  0->ISC10
  MCUCR |= _BV(ISC11); MCUCR &= ~_BV(ISC10);
  
  GICR  |= _BV(INT1);   //  INT1
  GICR  |= _BV(INT2);   //  INT2

  dataReady = 0; endFlag = 0;
  
  sei();
  for(;;){              //  
    wdt_reset();
    usbPoll();
  }
  return 0;
}
//=============================================================================
